/****************************************************************************
	Bagotronix
	1019 Crossing Brook Way
	Tallahassee, FL 32311

	(C) Copyright 1996 Bagotronix
	All rights reserved

	Filename:  		csfcadc.c
	Programmer(s):	Ivan Baggett
	Description:	Sunflower C(tm) library
					Chickadee(tm) A/D converter functions
	comments:
****************************************************************************/


#include "stdinc.h"
#include "chickade.h"
#include "csfcadc.h"


/****************************************************************************
LTC1296 and MAX186/188 channel multiplexor constant definitions
****************************************************************************/
static const char ADCchannel [32] = {
	/* single-ended channel selections (LTC1296) */
	0xc7,   /* CH0 */
	0xe7,   /* CH1 */
	0xcf,   /* CH2 */
	0xef,   /* CH3 */
	0xd7,   /* CH4 */
	0xf7,   /* CH5 */
	0xdf,   /* CH6 */
	0xff,   /* CH7 */
	/* differential channel selections (CH +#,-#) (LTC1296) */
	0x87,   /* CH 0,1 */
	0x8f,   /* CH 2,3 */
	0x97,   /* CH 4,5 */
	0x9f,   /* CH 6,7 */
	0xa7,   /* CH 1,0 */
	0xaf,   /* CH 3,2 */
	0xb7,   /* CH 5,4 */
	0xbf,   /* CH 7,6 */
	/* single-ended channel selections (MAX186/MAX188) */
	0x8e,   /* CH0 */
	0xce,   /* CH1 */
	0x9e,   /* CH2 */
	0xde,   /* CH3 */
	0xae,   /* CH4 */
	0xee,   /* CH5 */
	0xbe,   /* CH6 */
	0xfe,   /* CH7 */
	/* differential channel selections (CH +#,-#) (MAX186/MAX188) */
	0x8a,   /* CH 0,1 */
	0x9a,   /* CH 2,3 */
	0xaa,   /* CH 4,5 */
	0xba,   /* CH 6,7 */
	0xca,   /* CH 1,0 */
	0xda,   /* CH 3,2 */
	0xea,   /* CH 5,4 */
	0xfa    /* CH 7,6 */
};


/****************************************************************************
ADCget():  reads the ADC
		args:  device type (0=LTC1296, 1=MAX186), channel#,
			acquisition time (given in delay loop iterations)
		returns:  the ADC result, MSB justified, unsigned
		uses:  ADC, 16C452 parallel port
		comments:  see CHKdelay() and DLY() for details about timing
			works with both LTC1296 and MAX186, but use proper ADCchannel[]
			lookup table for chip being used
			written in x86 assembly for speed
		example:  i = ADCget (0, 2, DLY (100000));
					 { LTC1296, channel 2, 100 us acquisition time }
		verified:  7/9/96 MAX186 and LTC1296, single-ended channel selections
			only, 7/10/96 acquistion delays
****************************************************************************/
UWORD ADCget (UBYTE devicetype, UBYTE channel, UWORD atime) {
	_DI = atime / 3;			// distribute acq time amone 3 SCL's
	if (_DI == 0)				// ensure at least one delay iteration
		_DI = 1;
	_BL = ADCchannel [devicetype * 16 + channel];
	_BH = 0;                        // first bit is 0 before start condition
asm {
	mov     dx, LPTCONTROL
	in      al, dx
	or      al, LPTSTB          // SCL -> lo
	out     dx, al
	or      al, LPTAFD              // -CSADC -> lo; AFD bit inverted vs. pin
	out     dx, al
	mov     cx, 9                   // (# of bits + 1) to shift out
	}
ADCget_1:
asm {
	or      al, LPTSTB          // SCL -> lo
	out     dx, al
	test    bh, 0x01
	jnz     ADCget_2            // jmp if lsb of bh = 1
	or      al, LPTSLIN         // lsb of bh = 0; SLIN bit inverted vs. pin
	jmp     ADCget_3
	}
ADCget_2:
asm and     al, NOT LPTSLIN;
ADCget_3:
asm {
	out     dx, al
	and     al, NOT LPTSTB              // SCL -> hi
	out     dx, al
	shl     bx, 1               // put next bit into msb position
	cmp     cx, 3               // if last 3 bits, do acquistion delay
	jg      ADCget_3_1
}

	//  INSERT CALL TO DELAY HERE
asm push	cx;			// got to do this, since compiler cleans up stack
						// by popping cx with garbage
	CHKdelay (_DI);				// load cx with 1/3 acq time
asm	pop		cx;			// got to do this, since compiler cleans up stack
						// by popping cx with garbage

ADCget_3_1:
asm {
	loop    ADCget_1

	or      al, LPTSTB          // SCL -> lo
	out     dx, al
	and     al, NOT LPTSLIN     // ensure SDIO = 1 so SDI can funtion
	out     dx, al
	xor     bx, bx              // clear bx for incoming ADC result
	mov     cx, 13
	}
ADCget_4:
asm {
	shl     bx, 1               // put next bit into lsb position
	push    ax
	push    dx
	mov     dx, LPTSTATUS       // read DOUT pin of ADC
	in      al, dx
	test    al, LPTERROR        // test the received bit
	jz      ADCget_5
	or      bx, 1               // set lsb if bit = 1
	}
ADCget_5:
asm {
	pop     dx
	pop     ax
	and     al, NOT LPTSTB              // SCL -> hi
	out     dx, al
	or      al, LPTSTB                  // SCL -> lo
	out     dx, al
	loop    ADCget_4

	and     al, NOT LPTAFD          // -CSADC -> hi
	out     dx, al

	shl		bx, 4					// MSB-justify result
	}

	return _BX;
}

